# Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
#
# MultipartParser parses MIME-multipart parts.
#
# MultipartParser instances are file-like objects with additional
# method .next() which advances reading to the next part.
#
class Error(Exception):
    pass

class MalformedBoundaryError(Error):
    pass

class NoBoundaryFoundError(Error):
    pass


class MultipartParser(object):

    def __init__(self, fh, boundary):
        self._fh       = fh
        self._boundary = "\n--" + boundary
        self._boundary_len = len(self._boundary)
        self._buf      = ""
        self.__boundaryOccured = False
        self.__headersRead = True
        self.partSize = None

        # move to the first MIME part
        self.__skipToBoundary()
        if self.__skipBoundary():
            raise MalformedBoundaryError("Unexpected final boundary after preamble")

    def read(self, size):
        """
        Read not more than size bytes, until next mime part.
        Returns "" when MIME part ends
        """
        if self.__boundaryOccured:
            return ""

        # 1. There is not enough data in the buffer, to determine that there is no boundary
        # among size bytes. Readlines until we gather enough data.
        bytes_to_read = (size + self._boundary_len + 2) - len(self._buf)

        if bytes_to_read > 0:
            self._buf += self._fh.read(bytes_to_read)

        if not self.__headersRead:
            self.__headersRead = True
            self.partSize = None

            start = 0
            while True:
                end = self._buf.find("\n", start)

                if end <= start + 1:
                    break

                if self.partSize is None:
                    header = self._buf[start:end]
                    if header[:14] == "Content-Length":
                        self.partSize = int(header[16:])

                start = end + 1

            if self.partSize != None:
                self.partSize += start + 1

        if self.partSize != None:
            if self.partSize  <= len(self._buf) - (self._boundary_len + 2):
                boundary_start = self.partSize
                boundary = self._buf[boundary_start:boundary_start + self._boundary_len + 1]

                if boundary.lstrip("\r")[:self._boundary_len] == self._boundary:
                    self.__boundaryOccured = True
            else:
                boundary_start = size

            self.partSize -= boundary_start

        else:
            # 2. There is enough data in the buffer to make a decision (size + boundary_len)
            # Search boundary in buffer only
            boundary_start = self._buf.find(self._boundary)

            # if boundary is stright at the begin of the buffer - we have extracted previous
            # MIME part completely. Return empty string. next() will be than called by the callee

            # no boundary start among first size bytes, so return first size bytes of the buffer

            if boundary_start == -1 or boundary_start > size:
                boundary_start = size
            else:
                if boundary_start > 0 and self._buf[boundary_start - 1] == '\r':
                    boundary_start -= 1
                self.__boundaryOccured = True

        retbuf, self._buf = self._buf[:boundary_start], self._buf[boundary_start:]
        return retbuf

    def __skipToBoundary(self):
        if not self.__boundaryOccured:
            while self.read(65536):
                pass

    # skips boundary string and returns whether final boundary occured
    def __skipBoundary(self):
        self.__boundaryOccured = self.__headersRead = False

        finalBoundary = False

        l = self._boundary_len

        if self._buf[0] == '\r':
            l += 1

        if self._buf[l:l+2] == "--":
            l += 2
            finalBoundary = True
        else:
            if self._buf[l] == '\r':
                l += 1

        self._buf = self._buf[l + 1:] # skip \n

        return finalBoundary

    def close(self):
        # skip to the next mime part
        self.__skipToBoundary()

    def next(self):
        """
        Returns true when there is next MIME part
        Returns false when there are no MIME parts left
        """
        self.__skipToBoundary()
        if not self.__boundaryOccured:
            raise NoBoundaryFoundError("Premature EOF without final boundary")

        # skip boundary string and return True if it was not a finalBoundary
        return not self.__skipBoundary()


# Local Variables:
# mode: python
# indent-tabs-mode: nil
# tab-width: 4
# End:
