<?php
// Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
/**
 * Script for convertation of localization files from Plesk to SMB format
 */

/**
 * Return help message
 *
 * @return string
 */
function getHelp()
{
    $script = $GLOBALS['argv'][0];
    return <<<HELP
Usage: $script -f localeFile [-o outputDirectory] [-l languageCode]
-f localeFile       Plesk localization file
-o outputDirectory  output directory path (optional, current directory is used by default)
-l languageCode     language code (optional, utility tries to detect it from locale file name, en-US is used by default)
Example of usage: $script -f messages_en-US.php -o application/resources/languages -l de-DE

See http://wiki.plesk.ru/wiki/SMB_Translation#Interaction_with_the_localization_team for details

HELP;
}

/**
 * Return script options
 *
 * @return array
 */
function getOptions()
{
    $options = getopt('f:o:l:');
    if (false === $options || !array_key_exists('f', $options)) {
        throw new Exception(getHelp());
    } else {
        if (!array_key_exists('o', $options)) {
            $options['o'] = '.';
        }
        if (!array_key_exists('l', $options)) {
            $options['l'] = false;
        }
    }
    return $options;
}

/**
 * Run utility
 *
 */
function run()
{
    $options = getOptions();

    $localeFile = $options['f'];
    $outputPath = rtrim($options['o'], DIRECTORY_SEPARATOR);
    $lang = $options['l'];

    if (false === $lang) {
        //try to detect language by Plesk localization file name
        if (preg_match('/(\w\w-\w\w)\.php$/', $localeFile, $matches)) {
            $lang = $matches[1];
        } else {
            $lang = 'en-US';
        }
    }

    if (!file_exists($localeFile)) {
        throw new Exception("Plesk localization file '$localeFile' does not exist\n");
    }

    $translations = process($localeFile);
    output($translations, $outputPath, $lang);
}

/**
 * Process locale file into comfortable format
 *
 * @param string $localeFile
 * @return array
 */
function process($localeFile)
{
    $lines = file($localeFile);
    $translations = array();
    $success = true;
    foreach ($lines as $line) {
        if (false === strpos($line, '=>')) {
            continue ;
        }

        if (!preg_match('/^(\s|[^\'"])*[\'"]([^\'"]+)\./', $line, $matches)) {
            echo "Invalid line: $line";
            $success = false;
            continue;
        }

        $fileName = $matches[2];

        $line = preg_replace('/^((\s*|[^\'"])*[\'"])([^\'"]+\.)/', '\\1', $line);

        $translations[$fileName][] = $line;
    }

    if (false === $success) {
        throw new Exception("Cannot parse '$localeFile'\n");
    }

    return $translations;
}

/**
 * Create SMB translations storage
 * @see http://wiki.plesk.ru/wiki/SMB_Translation#Translations_Storage.
 *
 * @param array $translations array of <section> => array of <key> => <message>
 * @param string $outputDir
 * @param string $lang language code
 */
function output($translations, $outputDir, $lang)
{
    if (!is_dir($outputDir)) {
        throw new Exception("Output directory '$outputDir' does not exist\n");
    }

    $outputDir .= DIRECTORY_SEPARATOR . $lang;
    if (!is_dir($outputDir)) {
        mkdir($outputDir);
    }

    foreach ($translations as $section => $messages) {
        $sectionFile = str_replace('.', DIRECTORY_SEPARATOR, $section) . '.php';

        echo "File: $sectionFile\n";

        $dirs = explode(DIRECTORY_SEPARATOR, $sectionFile);
        array_pop($dirs);

        $dir = $outputDir;
        foreach ($dirs as $part) {
            $dir .= DIRECTORY_SEPARATOR . $part;
            if (!is_dir($dir)) {
                mkdir($dir);
            }
        }

        $content = render($messages);
        file_put_contents($outputDir . DIRECTORY_SEPARATOR . $sectionFile, $content);
    }

    echo "Locale $lang have been prepared.\n";
    echo "Directory: $outputDir\n";
}

/**
 * Return locale file content
 *
 * @param array $messages array <key> => <message>
 * @return string
 */
function render($messages)
{
    $header = <<<PHP
<?php

\$messages = array(

PHP;

    $footer = <<<PHP
);

PHP;

    return $header . join($messages) . $footer;
}

require_once(dirname(__FILE__) . '/getopt.php');

try {
    run();
} catch (Exception $e) {
    echo $e->getMessage();
    exit(1);
}
