# Copyright 1999-2014. Parallels IP Holdings GmbH. All Rights Reserved.
# ex: set ft=python:
import os
import shutil
import sqlite3
import filecmp
import plesk_log
from datetime import datetime
from glob import glob
from operator import itemgetter


log = plesk_log.getLogger(__name__)


def dict_factory(cursor, row):
    dictionary = {}
    for index, column in enumerate(cursor.description):
        dictionary[column[0]] = row[index]
    return dictionary


STORAGE_PATH = '/var/lib/plesk/cfgmon'
PROTECTED_FILELIST_FILENAME = 'protected_files'
TIMESTAMP_FORMAT = '%Y_%m_%d_%H_%M_%S_%f'
MAX_REVISIONS_FOR_FILE = 50
PROTECTED_FILE_MAX_SIZE = 1024 * 1000


class RotationException(Exception):
    pass


class StoringException(Exception):
    pass


class Formatter():
    @staticmethod
    def formatHistory(objectList):
        objectList.sort(key=itemgetter("id"))
        historyFormat = " %(id)s\t%(createDate)s\t%(filePath)s\t\t\t\t%(inStoragePath)s\n"
        output = 'Rev.\tTimeStamp\t\tFile path\t\t\t\tIn Storage\n'
        for obj in objectList:
            output += historyFormat % obj
        return output

    @staticmethod
    def formatList(objectList):
        objectList.sort(key=itemgetter("createDate"))
        listFormat = " %(createDate)s\t%(filePath)s\n"
        output = ' TimeStamp\t\tFile path\n'
        for obj in objectList:
            if 'deleted' in obj:
                obj['filePath'] = 'WARNING! Original file deleted. \t %s' % obj['filePath']
            output += listFormat % obj
        return output

    @staticmethod
    def formatFileList(fileList):
        output = 'List of protected files:\n'
        for obj in fileList:
            output += "\t%s\n" % obj
        return output


class PleskMetaDb():
    _connection = None
    _databaseCatalog = None
    _databaseName = 'meta.db'

    def __init__(self, databaseCatalog, autoinit=True):
        self._databaseCatalog = databaseCatalog
        if autoinit:
            self.init()

    def init(self):
        self.getConnection()

    def _perform(self, query, params=None):
        log.debug("Quering: '%s' with params %s" % (query, params))
        with self._connection as con:
            if params is None:
                output = con.execute(query)
            else:
                output = con.execute(query, params)
        return output

    def getDatabasePath(self):
        return os.path.join(self._databaseCatalog, self._databaseName)

    def getConnection(self):
        if not self._isMetaDbExists():
            self._createMetaDb()
        if self._connection is None:
            self._connection = sqlite3.connect(self.getDatabasePath())
            self._connection.row_factory = dict_factory
        return self._connection

    def insertMeta(self, filePath, inStoragePath, actor):
        query = """INSERT INTO meta (filePath, inStoragePath, actor, createDate)
            VALUES (?, ?, ?, datetime('now'));"""
        self._perform(query, (filePath, inStoragePath, actor))

    def getByFilePath(self, filePath):
        query = """SELECT * FROM meta WHERE filePath = ?"""
        return self._perform(query, (filePath,)).fetchall()

    def getById(self, id):
        query = """SELECT * FROM meta WHERE id = ?"""
        return self._perform(query, (id,)).fetchone()

    def getLastByFilePath(self, filePath):
        query = """SELECT * FROM meta WHERE filePath = ? ORDER BY createDate DESC LIMIT 1"""
        return self._perform(query, (filePath,)).fetchone()

    def getEarlyByFilePath(self, filePath):
        query = """SELECT * FROM meta WHERE filePath = ? ORDER BY createDate LIMIT 1"""
        return self._perform(query, (filePath,)).fetchone()

    def removeById(self, id):
        query = """DELETE FROM meta WHERE id = ?"""
        return self._perform(query, (id,))

    def getMonitoredFiles(self):
        query = """SELECT DISTINCT filePath FROM meta"""
        return self._perform(query).fetchall()

    def _isMetaDbExists(self):
        if os.path.exists(self.getDatabasePath()):
            log.debug('Meta database "%s" exists', self.getDatabasePath())
            return True
        log.debug('Meta database "%s" doesn\'t exist', self.getDatabasePath())
        return False

    def _createMetaDb(self):
        log.debug('Try to create meta database and table')
        self._connection = sqlite3.connect(self.getDatabasePath())
        query = """CREATE TABLE meta (
            id              INTEGER PRIMARY KEY AUTOINCREMENT,
            filePath        VARCHAR(255),
            inStoragePath   VARCHAR(255),
            actor           VARCHAR(32),
            createDate      DATETIME);"""
        self._connection.execute(query)
        self._connection.commit()
        self._connection.close()
        self._connection = None
        log.debug('Table was created')


class PleskFileStorage():
    """
        Object of this class contains all hilevel logic to store files,
        list stored files and list stored files revisions
    """
    def __init__(self, storagePath = None):
        log.debug('Try to init storage')
        self._storagePath = STORAGE_PATH
        self._db = None
        self._actor = 'sysUser'

        if storagePath is not None:
            self._storagePath = storagePath

        if not self._isStorageExists():
            self._createStorage(self.getStoragePath())
        self._db = PleskMetaDb(self.getStoragePath())

    def getStoragePath(self):
        return self._storagePath

    def setActor(self, actor='sysUser'):
        log.debug('Set actor to "%s"', actor)
        self._actor = actor

    def getActor(self):
        return self._actor

    def storeFile(self, filePath):
        log.debug('Storing file "%s"', filePath)
        try:
            if os.path.isdir(filePath):
                log.debug('%s is directory', filePath)
                return None
            if os.path.islink(filePath):
                log.debug('%s is symlink', filePath)
                return None
            if not self.isFileChanged(filePath):
                log.debug('File not changed. No need to store "%s"', filePath)
                return None

            statinfo = os.stat(filePath)
            if statinfo.st_size > PROTECTED_FILE_MAX_SIZE:
                log.warn('File "%s" too big to store. Actual size: %s, max: %s',
                        filePath, statinfo.st_size, PROTECTED_FILE_MAX_SIZE)
                return None

            newFilePath = self._generateNewFilePath(filePath)

            log.debug('Try to copy "%s" to "%s"', filePath, newFilePath)
            shutil.copyfile(filePath, newFilePath)
            log.debug('File copied')
            self._db.insertMeta(filePath, newFilePath, self.getActor())
            print 'File "%s" was stored successfully.' % (filePath)
            return newFilePath
        except Exception, e:
            raise StoringException('Unable to store %s: %s' % (filePath, e))

    def isFileChanged(self, filePath, fileDict=None):
        log.debug('Check if file %s is changed', filePath)
        if fileDict is None:
            fileDict = self._getLastRevision(filePath)
        if fileDict is None or fileDict['id'] is None:
            log.debug('There is no information about %s in metadata', filePath)
            return True
        if not os.path.exists(fileDict['inStoragePath']):
            return True
        isEqual = filecmp.cmp(fileDict['filePath'], fileDict['inStoragePath'])
        if isEqual:
            log.debug('File %s equal %s', fileDict['filePath'], fileDict['inStoragePath'])
        else:
            log.debug('File %s not equal %s', fileDict['filePath'], fileDict['inStoragePath'])
        return not isEqual

    def storeAllFiles(self):
        log.debug('Store all configuration files')
        failedFlag = False
        isSomeWorkPerformed = False
        for filePath in self._getProtectedFilesList():
            log.debug('Perform rotation and storing of "%s"', filePath)
            if os.path.islink(filePath):
                log.debug('Skip link processing "%s"', filePath)
                continue
            try:
                isSomeWorkPerformed = (self._rotateFile(filePath) != None or isSomeWorkPerformed)
                isSomeWorkPerformed = (self.storeFile(filePath) != None or isSomeWorkPerformed)
            except RotationException, e:
                failedFlag = True
                log.error(e)
            except StoringException, e:
                failedFlag = True
                log.error(e)
            log.debug('[%s] Done.\n', filePath)

        if failedFlag:
            raise Exception('Unable to rotate or store some files')

        if not isSomeWorkPerformed:
            print 'No need to store or rotate any files.'

    def getAllFilesStats(self):
        output = []
        monitored = self._db.getMonitoredFiles()
        monitored = map(lambda item: item['filePath'], monitored)
        log.debug('Monitored files: %s', monitored)
        for config in self._getProtectedFilesList():
            if config in monitored:
                monitored.remove(config)
            lastRevision = self._getLastRevision(config)
            if lastRevision is None or lastRevision['filePath'] is None:
                continue
            output.append(lastRevision)

        if len(monitored) != 0:
            log.debug('There are some deleted configs %s' % config)
            for config in monitored:
                lastRevision = self._getLastRevision(config)
                lastRevision['deleted'] = True
                output.append(lastRevision)
        return output

    def getFileHistory(self, filePath):
        log.debug('Try to get history of "%s"', filePath)
        data = self._db.getByFilePath(filePath)
        if len(data) == 0:
            log.warn('No information about "%s" in storage', filePath)
            print Formatter.formatFileList(self._getProtectedFilesList())
        return data

    def getLastFileRevisionByFilePath(self, filePath):
        fileDict = self._db.getLastByFilePath(filePath)
        if fileDict is None:
            log.warn('No information about file path "%s" in storage', filePath)
            exit(1)
        return self._readFile(fileDict['inStoragePath'])

    def getFileByRevision(self, id):
        fileDict = self._db.getById(id)
        if fileDict is None:
            log.warn('No information about revision with id "%s" in storage', id)
            exit(1)
        return self._readFile(fileDict['inStoragePath'])

    def _readFile(self, filePath):
        with open(filePath, 'r') as storedFile:
            return storedFile.read()

    def removeFromStorage(self, fileDict):
        log.debug('Remove "%s" metadata from database', fileDict['inStoragePath'])
        self._db.removeById(fileDict['id'])
        log.debug('Remove "%s" from storage', fileDict['inStoragePath'])
        os.unlink(fileDict['inStoragePath'])

    def _rotateFile(self, filePath):
        log.debug('Try to rotate file %s', filePath)
        try:
            revisions = self._db.getByFilePath(filePath)

            revisionCount = len(revisions)
            if revisionCount < MAX_REVISIONS_FOR_FILE:
                log.debug('No need to rotate file "%s", revisions=%s, max=%s',
                        filePath, revisionCount, MAX_REVISIONS_FOR_FILE)
                return
            log.debug('Rotate file %s, revisions=%s, max=%s', filePath,
                        revisionCount, MAX_REVISIONS_FOR_FILE)
            early = self._db.getEarlyByFilePath(filePath)
            self.removeFromStorage(early)
        except Exception, e:
            raise RotationException('Unable to rotate file %s: %s'
                    % (filePath, e))

    def _getProtectedFilesList(self):
        filePath = os.path.join(self._storagePath, PROTECTED_FILELIST_FILENAME)
        fileList = []
        if os.path.exists(filePath):
            log.debug('%s exists try to read...', filePath)
            with open(filePath, 'r') as fileObj:
                for line in fileObj.readlines():
                    line = line.strip()
                    if line == '':
                        continue
                    if line[0] == '#':
                        continue
                    fileList += glob(line)
        else:
            log.error('%s doesn\'t exist', filePath)
            exit(1)
        log.debug('Configuration file list is %s', fileList)
        return fileList

    def _formatOutput(self, fileDict):
        print 'Revision %s\t%s\t modified by %s' % (
                fileDict['id'], fileDict['filePath'], fileDict['actor'])

    def _getLastRevision(self, filePath):
        log.debug('Get last revision for "%s"', filePath)
        if os.path.isdir(filePath):
            log.debug('"%s" is a directory.', filePath)
            return None
        return self._db.getLastByFilePath(filePath)

    def _isStorageExists(self):
        if os.path.exists(self.getStoragePath()):
            log.debug('Storage path "%s" exists', self.getStoragePath())
            return True
        log.debug('Storage path "%s" doesn\'t exist', self.getStoragePath())
        return False

    def _createStorage(self, path):
        log.debug('Try to create storage "%s"', self.getStoragePath())
        os.makedirs(path, 0700)

    def _generateNewFilePath(self, filePath):
        log.debug('Generating new filepath for file "%s"', filePath)
        fileName = os.path.basename(filePath)

        genFile = "%s_%s" % (fileName, self._getTimeStamp())
        return os.path.join(self.getStoragePath(), genFile)

    def _getTimeStamp(self):
        return datetime.now().strftime(TIMESTAMP_FORMAT)

